//杨辉三角（也称帕斯卡三角），是二项式系数的一种三角形几何排列，它存在以下性质：
//1 第n 行的第m 个数和第n-m+1 个数相等，为组合数C(n,m)  (n, m 从0开始计数)。
//2 三角形两腰均为1，三角形内部数等于其上一行相邻两数之和。
//杨辉三角形实例
// 1             n = 0
// 1 1           n = 1
// 1 2 1         n = 2
// 1 3 3 1       n = 3
// 1 4 6 4 1     n = 4

//本函数生成杨辉三角形的前n行（n < 30）的值，并储存在二维数组a中

#include<limits.h>

#define N 30      //当 n > 33 时，生成的最大数会超出整型（int）的范围
unsigned a[N][N] = {0};

// 逻辑函数 pow(x)计算2的x次幂。它用于给出计算中的 a[x][y] 一个上界，说明其值不会产生数值上溢
/*@
    logic unsigned pow(unsigned n) = n <= 0 ? 1 : 2*pow(n-1);
*/

//////////////////////////////////////yanghui_triangle函数的验证////////////////////////////////////
//函数协议
//  前条件：说明n 的取值范围和数组a的初值；
//  后条件：描述了杨辉三角形的构成特性。
//关键字：循环不变式，数值溢出，assigns子句

/*@
  requires
      1 <= n < N;
  assigns a;
  ensures
      (\forall integer x:[0..n-1].a[x][0] == a[x][x] == 1)
      && (\forall integer x:[2..n-1].\forall integer y:[2..x-1].a[x][y] == a[x-1][y] + a[x-1][y-1])
      ;
*/
void yanghui_triangle(const unsigned n)
{
    unsigned i, j;  
    /*@ 
      loop invariant
          0 <= i <= n 
          && (\forall integer x:[0..i-1].a[x][0] == a[x][x] == 1)
          &&  1 <= n < N  //该断言来自函数前条件
          ;
    */
    
    for(i=0; i<n; i++){  // 循环从0开始，则a[0][0]为第一个数
        a[i][0] = 1; a[i][i] = 1;  // 设定杨辉三角形两腰的数值为1
    }  
    
    /*@ lemma limit_of_power_n:  // 引理用于说明pow(n)在计算过程中不会出现数值上溢
            \forall unsigned n. 0 < n < 30 ==> pow(n) >= 1 && pow(n) <= UINT_MAX;
    */
    /*@ 
      loop invariant
          (i == 1 || 1 < i <= n ) 
          && (\forall integer x:[2..i-1].\forall integer y:[1..x-1].a[x][y] == a[x-1][y] + a[x-1][y-1])
          && (\forall integer x:[0..i-1].\forall integer y:[0..x]. a[x][y] <= pow(x) )
          //下列断言来自上个for循环
          && 1 <= n < N
          && (\forall integer x:[0..n-1].a[x][0] == a[x][x] == 1)
          ;  
    */
    for(i=1; i<n; i++){
        /*@ 
          loop invariant
              1 <= j <= i 
              && (\forall integer y:[1..j-1].a[i][y] == a[i-1][y] + a[i-1][y-1])
              && (\forall integer y:[1..j-1].a[i][y] <= pow(i))
              //下列断言来自外循环
              && 1 <= i < n 
              && (\forall integer x:[2..i-1].\forall integer y:[1..x-1].a[x][y] == a[x-1][y] + a[x-1][y-1])
              && (\forall integer x:[0..i-1].\forall integer y:[0..x]. a[x][y] <= pow(x) )
              && 1 <= n < N
              && (\forall integer x:[0..n-1].a[x][0] == a[x][x] == 1)
              ;
        */
        for(j=1; j<i; j++){
            a[i][j] = a[i-1][j-1] + a[i-1][j];  //除两边的数外，杨辉三角形的值都等于上一行相邻两数之和
        }
    }
}